"""Constants for DirecTV."""

VALID_REMOTE_KEYS = [
    "power",
    "poweron",
    "poweroff",
    "format",
    "pause",
    "rew",
    "replay",
    "stop",
    "advance",
    "ffwd",
    "record",
    "play",
    "guide",
    "active",
    "list",
    "exit",
    "back",
    "menu",
    "info",
    "up",
    "down",
    "left",
    "right",
    "select",
    "red",
    "green",
    "yellow",
    "blue",
    "chanup",
    "chandown",
    "prev",
    "0",
    "1",
    "2",
    "3",
    "4",
    "5",
    "6",
    "7",
    "8",
    "9",
    "dash",
    "enter",
]
